'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.subscribeHandler = subscribeHandler;
exports.unsubscribeHandler = unsubscribeHandler;
exports.mergeStateHandler = mergeStateHandler;
exports.provideViewModelsHandler = provideViewModelsHandler;
exports.createMap = createMap;
exports.viewModelEventHandler = viewModelEventHandler;
exports.default = createViewModelsReducer;

var _actions = require('./actions');

var _util = require('./util');

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function subscribeHandler(subscribers, viewModels, state, _ref) {
    var viewModel = _ref.viewModel,
        aggregateId = _ref.aggregateId;

    var key = (0, _util.getKey)(viewModel, aggregateId);

    if (subscribers[key]) {
        subscribers[key]++;
        return state;
    }

    subscribers[key] = 1;

    return _extends({}, state, _defineProperty({}, viewModel, _extends({}, state[viewModel], _defineProperty({}, aggregateId, (viewModels.find(function (_ref2) {
        var name = _ref2.name;
        return viewModel === name;
    }).projection.Init || function () {})()))));
}

function unsubscribeHandler(subscribers, state, _ref3) {
    var viewModel = _ref3.viewModel,
        aggregateId = _ref3.aggregateId;

    var key = (0, _util.getKey)(viewModel, aggregateId);

    if (subscribers[key] > 1) {
        subscribers[key]--;
        return state;
    }

    subscribers[key] = 0;

    var _state$viewModel = state[viewModel],
        _ = _state$viewModel[aggregateId],
        nextViewModel = _objectWithoutProperties(_state$viewModel, [aggregateId]);

    return _extends({}, state, _defineProperty({}, viewModel, nextViewModel));
}

function mergeStateHandler(state, _ref4) {
    var viewModel = _ref4.viewModel,
        aggregateId = _ref4.aggregateId,
        actionState = _ref4.state;

    return _extends({}, state, _defineProperty({}, viewModel, _extends({}, state[viewModel], _defineProperty({}, aggregateId, actionState))));
}

function provideViewModelsHandler(initialState, handlers, subscribers, state, _ref5) {
    var viewModels = _ref5.viewModels;

    delete handlers[_actions.PROVIDE_VIEW_MODELS];

    handlers[_actions.SUBSCRIBE] = subscribeHandler.bind(null, subscribers, viewModels);

    handlers[_actions.UNSUBSCRIBE] = unsubscribeHandler.bind(null, subscribers);

    handlers[_actions.MERGE_STATE] = mergeStateHandler;

    viewModels.forEach(function (_ref6) {
        var viewModel = _ref6.name;

        initialState[viewModel] = {};
    });

    var map = createMap(viewModels);

    Object.keys(map).forEach(function (eventType) {
        handlers[eventType] = viewModelEventHandler.bind(null, map, eventType);
    });

    return initialState;
}

function createMap(viewModels) {
    return viewModels.reduce(function (acc, _ref7) {
        var viewModel = _ref7.name,
            _ref7$projection = _ref7.projection,
            Init = _ref7$projection.Init,
            projection = _objectWithoutProperties(_ref7$projection, ['Init']);

        Object.keys(projection).forEach(function (eventType) {
            if (!acc[eventType]) {
                acc[eventType] = {};
            }

            acc[eventType][viewModel] = projection[eventType];
        });
        return acc;
    }, {});
}

function viewModelEventHandler(map, eventType, state, action) {
    var nextState = _extends({}, state);

    Object.keys(map[eventType]).forEach(function (viewModel) {
        if (!state[viewModel]) return;

        if (state[viewModel].hasOwnProperty('*')) {
            var _viewModelState = state[viewModel]['*'];

            var _result = map[eventType][viewModel](_viewModelState, action);

            nextState[viewModel] = _extends({}, nextState[viewModel], {
                '*': _result
            });
        }

        if (!state[viewModel].hasOwnProperty(action.aggregateId)) return;

        var viewModelState = state[viewModel][action.aggregateId];

        var result = map[eventType][viewModel](viewModelState, action);

        nextState[viewModel] = _extends({}, nextState[viewModel], _defineProperty({}, action.aggregateId, result));
    });

    return nextState;
}

function createViewModelsReducer() {
    var initialState = {};
    var handlers = {};
    var subscribers = {};

    handlers[_actions.PROVIDE_VIEW_MODELS] = provideViewModelsHandler.bind(initialState, handlers, subscribers);

    return function () {
        var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
        var action = arguments[1];

        var eventHandler = handlers[action.type];

        if (eventHandler) {
            return eventHandler(state, action);
        }

        return state;
    };
}